# Quick start

Code.Movie is a JavaScript library that **turns code snippets into animations** built from HTML and CSS. Think git diffs meets syntax highlighting meets CSS transitions. To get to a bit of basic animated JSON like this...

!!!json(|meta={ autoplay: true })

```()
[]
```

```()
["World"]
```

```()
["Hello", "World"]
```

```()
[
  "Hello",
  "World"
]
```

!!!

...your first declare your keyframes as a list of strings...

```javascript
let keyframes = [
  { code: `[]` }, // animate from this...
  { code: `["World"]` }, // ... to this ...
  { code: `["Hello", "World"]` }, //  ... to this ...
  { code: `[\n  "Hello",\n  "World"\n]` }, // ... and end with this
];
```

... then load [the core library](https://www.npmjs.com/package/@codemovie/code-movie), a [theme](./reference/Themes.md) and the language module, then pass the keyframes to a function:

```javascript
let keyframes = [
  /* keyframes go here */
];

import { animateHTML, monokaiDark } from "@codemovie/code-movie";
import json from "@codemovie/code-movie/languages/json";

// HTML = string of bog-standard HTML and CSS
let html = animateHTML(keyframes, {
  tabSize: 2,
  language: json(),
  theme: monokaiDark,
});
```

The end result is **animated, syntax highlighted code** ready for your next slide deck, coding tutorial, or other project needing to visualize code changes. If you want to get ambitious, you can also add a wide variety of decoration to highlight important snippets or direct your audiences attention:

!!!json(|meta={ autoplay: true })

```(|decorations=[{ kind: "GUTTER", text: "➡️", line: 1 }])
[]
```

```(|decorations=[{ kind: "TEXT", from: 1, to: 8 }])
["World"]
```

```(|decorations=[
  { kind: "TEXT", from: 1, to: 8 },
  { kind: "TEXT", from: 10, to: 17, data: { class: "error" } }
])
["Hello", "World"]
```

```(|decorations=[
  { kind: "GUTTER", text: "✅", line: 2 },
  { kind: "GUTTER", text: "🚫", line: 3 }
])
[
  "Hello",
  "World"
]
```

!!!

Things to note:

- The currently visible code can be **selected and copied** easily.
- Because this library inputs and outputs plain strings, you can generate the animation **in browsers as well as server-side or at compile-time.**
- Animations, once generated, require **no runtime JavaScript** and work entirely by switching classes to trigger different css states. Integrations like [`<code-movie-runtime>`](https://www.npmjs.com/package/@codemovie/code-movie-runtime) turn even this last step into a no-brainer.
- The animation's look can be customized with a _ton_ of [CSS variables](./reference/CSS-Variables.md). Several built-in [themes](./reference/Themes.md) are at your disposal.
- Animations can be generated [from markdown](https://www.npmjs.com/package/@codemovie/code-movie-marked-plugin)... just like on this page!
- Even though the animations are rendered using HTML and CSS, [turning them into HQ video is just a matter of running a simple script](https://code.movie/blog/animated-syntax-highlighting-4k-60-fps-edition.html).
- You can augment your animations with [decorations](./guides/Decorations.md) to underline, highlight or obscure select sections of code

This obviously skipped over a ton of details, but captures the gist of Code Movie. Check out [the full tutorial](./guides/Tutorial.md) for a more in-depth explanation of the above steps!

<aside>

## Peter's take

Hi! I'm Peter, the guy responsible for this project. You will see me put forward my personal take on some aspects of the project from time to time in boxes like this. These takes will not necessarily apply to _every_ possible use case, but will reflect my personal experience with using this piece of software (and its predecessors) over the last 7 years. If you have any questions, contact my via [email](mailto:peter@peterkroener.de), [Mastodon](https://mastodon.social/@sir_pepe), [Bluesky](https://bsky.app/profile/sirpepe.bsky.social) or [LinkedIn](https://www.linkedin.com/in/peterkroener/).

</aside>

Apart from going through the tutorial, you might want to read up on [the _many_ CSS variables](./guides/Styling.md) that you can change to tweak your animations. If you want to discuss something or want to report a bug, [head over to the CodeMovie meta repository on Github](https://github.com/CodeMovie/meta).
