# Decorations

[See the CSS reference](./CSS-Variables.md) for a list of CSS variables available for styling.

## Overview

| `kind`     | Use Case                 | Size/Placement                        | Equality        |
| ---------- | ------------------------ | ------------------------------------- | --------------- |
| `"GUTTER"` | Draw gutter icon         | Line number                           | `data` + `text` |
| `"LINE"`   | Decorate entire line(s)  | From line number to line number       | `data`          |
| `"TEXT"`   | Decorate section of code | From unicode offset to unicode offset | `data`          |

## Gutter decorations

Renders a single unicode character somewhere near the line number column.

```typescript
type InputGutterDecoration = {
  readonly kind: "GUTTER";
  line: number; // where to render the decoration
  text: string; // which character to render
  data: Record<string, string>;
};
```

Two gutter decorations count as equal when their `data` and `text` fields contain equal values. The built-in themes do not provide any presets for gutter decorations.

## Line decorations

Renders up to two boxes (one in the text foreground, one in the background) across one or more entire lines.

```typescript
type InputLineDecoration = {
  readonly kind: "LINE";
  fromLine: number; // line number to start at
  toLine: number; // line number to end at
  data: Record<string, string>;
};
```

Two line decorations count as equal when their `data` fields contain equal values. The built-in themes provide the presets `ok` and `error` for line decorations.

```plaintext(|decorations=[
  { kind: "LINE", fromLine: 1, toLine: 1, data: {} },
  { kind: "LINE", fromLine: 2, toLine: 2, data: { class: "ok" } },
  { kind: "LINE", fromLine: 3, toLine: 3, data: { class: "error" } }
])
Default line decoration
"ok" line decoration
"error" line decoration
```

## Text decorations

Decorate specific sections of code with backgrounds and/or underlines. Every text decoration renders up to _four_ elements per decorated section. Going from the background to the foreground this can include:

1. A background backdrop layer
2. A background underline layer
3. A foreground backdrop layer
4. A foreground underline layer

The highlighted code is sandwiched between the background and foreground layers.

Text decorations follow the following type definition:

```typescript
type InputGutterDecoration = {
  readonly kind: "TEXT";
  from: number; // where to start the decoration (code point offset)
  to: number; // where to end the decoration (code point offset)
  data: Record<string, string>;
};
```

Two text decorations count as equal when their `data` fields contain equal values. The The built-in themes provide the presets `ok` and `error` for text decorations.

```plaintext(|decorations=[
  { kind: "TEXT", from: 0, to: 23, data: {} },
  { kind: "TEXT", from: 24, to: 44, data: { class: "ok" } },
  { kind: "TEXT", from: 45, to: 68, data: { class: "error" } },
])
Default text decoration
"ok" text decoration
"error" text decoration
```
